///////////////////////////////////////////////////////////
//
// CS525 project 1 
// Suyu Zhang
// 
// Directly write on OpenCV and GLSL demo program from Andy Johnson 12/11
// originally based off the Lighthouse 3D tutorials
// bump part is copied from lecture's demo
///////////////////////////////////////////////////////////

#include "GL/glew.h"

#ifdef __linux__
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <glut.h>
#endif

#ifdef _WIN32
#include "glut.h"
#endif

#include "cv.h"
#include "highgui.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <time.h>

#include<ctime> 
using namespace std;

float mouseWindowX = 0.0;
float mouseWindowY = 0.0;

float bSize    = 0.15;
float bDensity = 16.0;

float a1 = 1.0;
float a2 = 1.0;
float a3 = 1.0;

#define IMAGE_SIZE 800

int winWidth = IMAGE_SIZE;
int winHeight = IMAGE_SIZE;

int flag = 0;

GLuint chosenTex, frame;

// openCV stuff
IplImage *image = 0;
IplImage *destination = 0;
IplImage * showMe;

GLuint cameraImageTextureID;
CvCapture *capture = 0;
GLuint noise3DTexName;
int noise3DTexSize = 128;
GLubyte *noise3DTexPtr;

////////////////////////////////////////////////////////////////////////////////

char *textFileRead(char *fn)
{
	FILE *fp;
	char *content = NULL;

	int count=0;

	if (fn != NULL) {
		fp = fopen(fn,"rt");

		if (fp != NULL) {
      
			fseek(fp, 0, SEEK_END);
			count = ftell(fp);
			rewind(fp);

			if (count > 0) {
				content = (char *)malloc(sizeof(char) * (count+1));
				count = fread(content,sizeof(char),count,fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	
	if (content == NULL)
	   {
	   fprintf(stderr, "ERROR: could not load in file %s\n", fn);
	   exit(1);
	   }
	return content;
}                    

//decide FPS
double CalFrequency()
{
    static int count;
    static double save;
    static clock_t last, current;
    double timegap;
    
    ++count;
    if( count <= 50 )
        return save;
    count = 0;
    last = current;
    current = clock();
    timegap = (current-last)/(double)CLK_TCK;
    save = 50.0/timegap;
    return save;
}
////////////////////////////////////////////////////////////////////////////////

GLuint p, p2;

float lpos[4] = {1.0, 0.5, 1.0, 0.0};

void changeSize(int w, int h) {

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	
	if(h == 0)
		h = 1;

	float ratio = 1.0* winWidth / winHeight;

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	winWidth = w; //winWidth;
	winHeight = h; // winWidth;
	
	// Set the viewport to be the entire window
    glViewport(0, 0, winWidth, winHeight);

	// Set the correct perspective.
	gluPerspective(45, ratio, 1, 1000);
	glMatrixMode(GL_MODELVIEW);
}
	
////////////////////////////////////////////////////////////////////////////////



void printShaderLog(GLuint prog)
{
    GLint infoLogLength = 0;
    GLsizei charsWritten  = 0;
    GLchar *infoLog;

    glGetShaderiv(prog, GL_INFO_LOG_LENGTH, &infoLogLength);

    if (infoLogLength > 0)
    {
        infoLog = (char *) malloc(infoLogLength);
        glGetShaderInfoLog(prog, infoLogLength, &charsWritten, infoLog);
		printf("%s\n",infoLog);
        free(infoLog);
    }
}

void printProgramLog(GLuint shad)
{
    GLint infoLogLength = 0;
    GLsizei charsWritten  = 0;
    GLchar *infoLog;

    glGetProgramiv(shad, GL_INFO_LOG_LENGTH, &infoLogLength);

    if (infoLogLength > 0)
    {
        infoLog = (char *) malloc(infoLogLength);
        glGetProgramInfoLog(shad, infoLogLength, &charsWritten, infoLog);
		printf("%s\n",infoLog);
        free(infoLog);
    }
}


////////////////////////////////////////////////////////////////////////////////

int which=0;
int mode=1;

void renderScene(void) {
    
    double FPS = CalFrequency();
    //printf("FPS = %f\n", FPS);
    
    GLint texLoc, effectLoc;//32位整数
    int xgrid, ygrid, counter;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(0.0,0.0,5.0, 
		      0.0,0.0,-1.0,
			  0.0f,1.0f,0.0f);
	glColor3f(1.0, 1.0, 1.0);	
	
    glLightfv(GL_LIGHT0, GL_POSITION, lpos);
    
    
    // openCV stuff
    IplImage *frame = 0;

    frame = cvQueryFrame( capture );

	// If I can't get a frame then exit the program
	if(!frame) 
		{
		exit(1);
		}

	// create a guaranteed texture size
	destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
	cvResize(frame, destination);

	// set the texture parameters
	glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	showMe = destination;

	if(showMe->nChannels == 3)
		{
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);
		} 
	else if(frame->nChannels == 4)
		{
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
		}

	glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
	

        
	// use the shader
	glUseProgram(p);
	
    GLint texLoc2, lightLoc, bumpSizeLoc, bumpDensityLoc;
    texLoc2 = glGetAttribLocation(p, "Tangent");
    glVertexAttrib3f(texLoc2, 1.0, 0.0, 0.0);
    
    
    
    lightLoc = glGetUniformLocation(p, "LightPosition");
    glUniform3f(lightLoc, 1.0, 1.0, 4.0);
    
    bumpSizeLoc = glGetUniformLocation(p, "BumpSize");
    glUniform1f(bumpSizeLoc, bSize);
    
    bumpDensityLoc = glGetUniformLocation(p, "BumpDensity");
    glUniform1f(bumpDensityLoc, bDensity);
    
    glNormal3f(0.0f, 0.0f, 1.0f );
    
    
	// using texture 0
    texLoc = glGetUniformLocation(p, "whichTexture");
    glUniform1i(texLoc, 0);


	// by default draw a white polygon if the texturing doesn't work.
    glColor3f(1.0, 1.0, 1.0);
    counter = 0;
    
    // draw the textured polygons
    // for some more sophisitcated effects you will probably want to render the texture to an offscreen
    // buffer first to be able to chain multiple shader effects together

    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 0);
    //上左
    glBegin( GL_POLYGON );	
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( -2.0, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  0.0f, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  0.0f,  2.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( -2.0f,  2.0f, 0.0f );
    glEnd();//map the texture to polygon
    //glPopMatrix();
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 1);
    //上右
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( 0.0f, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  2.0f, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  2.0f,  2.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( 0.0f,  2.0f, 0.0f );
    glEnd();//map the texture to polygon
    
    
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 2);
    
    //下左
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( -2.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  0.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  0.0f,  0.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( -2.0f,  0.0f, 0.0f );
    glEnd();//map the texture to polygon

    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 3);
    
    //下右
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( 0.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  2.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  2.0f,  0.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( 0.0f,  0.0f, 0.0f );
    glEnd();//map the texture to polygon
    
    
    
    /*for (xgrid=0; xgrid < 2; xgrid++)
    	for (ygrid=0; ygrid < 2; ygrid++)
			{
				// pick the effect to use
				effectLoc = glGetUniformLocation(p, "whichEffect");
				glUniform1i(effectLoc, counter);
				counter++;

			// draw the polygon with the current texture and effect
		
			glBegin( GL_POLYGON );    
				// GL_TEXTURE_RECTANGLE_ARB uses pixel coordinates rather than the standard OpenGL 0.0 to 1.0
				//glTexCoord2f( 0.0f, 1.0f );
				glTexCoord2i(0, showMe->height);
				glVertex3f( -2.0f+2*xgrid, -2.0f+2*ygrid, 0.0f );
				
				//glTexCoord2f( 1.0f, 1.0f ); 
				glTexCoord2i(showMe->width, showMe->height);
				glVertex3f(  0.0f+2*xgrid, -2.0f+2*ygrid, 0.0f );
				
				//glTexCoord2f( 1.0f, 0.0f );
				glTexCoord2i(showMe->width, 0);
				glVertex3f(  0.0f+2*xgrid,  0.0f+2*ygrid, 0.0f );
				
				//glTexCoord2f( 0.0f, 0.0f ); 
				glTexCoord2i(0, 0);
				glVertex3f( -2.0f+2*xgrid,  0.0f+2*ygrid, 0.0f );
		   glEnd();
		}*/
	glutSwapBuffers();
}


void renderScene2(void) {
    double FPS = CalFrequency();
    //printf("FPS = %f\n", FPS);
    GLint texLoc, effectLoc;//32位整数
    int xgrid, ygrid, counter;
    
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
	glLoadIdentity();
	gluLookAt(0.0,0.0,5.0, 
		      0.0,0.0,-1.0,
			  0.0f,1.0f,0.0f);
	glColor3f(1.0, 1.0, 1.0);	
	
    // openCV stuff
    IplImage *frame = 0;
    
    frame = cvQueryFrame( capture );
    
	// If I can't get a frame then exit the program
	if(!frame) 
    {
		exit(1);
    }
    
	// create a guaranteed texture size
	destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
	cvResize(frame, destination);
    
	// set the texture parameters
	glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    
	showMe = destination;
    
	if(showMe->nChannels == 3)
    {
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
    } 
	else if(frame->nChannels == 4)
    {
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
    }
    
	glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
	
    
    
	// use the shader
	glUseProgram(p);
	
    GLint texLoc2, lightLoc, bumpSizeLoc, bumpDensityLoc;
    texLoc2 = glGetAttribLocation(p, "Tangent");
    glVertexAttrib3f(texLoc2, 1.0, 0.0, 0.0);
    
    
    
    lightLoc = glGetUniformLocation(p, "LightPosition");
    glUniform3f(lightLoc, a1, a2, a3);
    
    bumpSizeLoc = glGetUniformLocation(p, "BumpSize");
    glUniform1f(bumpSizeLoc, bSize);
    
    bumpDensityLoc = glGetUniformLocation(p, "BumpDensity");
    glUniform1f(bumpDensityLoc, bDensity);
    
    glNormal3f(0.0f, 0.0f, 1.0f );
    
	// always using texture 0 for now
    texLoc = glGetUniformLocation(p, "whichTexture");
    glUniform1i(texLoc, 0);
    
    
	// by default draw a white polygon if the texturing doesn't work.
    glColor3f(1.0, 1.0, 1.0);
    counter = 0;
    
    // draw the textured polygons
    // for some more sophisitcated effects you will probably want to render the texture to an offscreen
    // buffer first to be able to chain multiple shader effects together
    
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 0);
    //上左
    glBegin( GL_POLYGON );	
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( -2.0, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  0.0f, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  0.0f,  2.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( -2.0f,  2.0f, 0.0f );
    glEnd();//map the texture to polygon
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 5);
    //上右
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( 0.0f, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  2.0f, 0.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  2.0f,  2.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( 0.0f,  2.0f, 0.0f );
    glEnd();//map the texture to polygon
    
    
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 6);
    
    //下左
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( -2.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  0.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  0.0f,  0.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( -2.0f,  0.0f, 0.0f );
    glEnd();//map the texture to polygon
    
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 7);
    
    //下右
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( 0.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  2.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  2.0f,  0.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( 0.0f,  0.0f, 0.0f );
    glEnd();//map the texture to polygon
    
    
    
    /*for (xgrid=0; xgrid < 2; xgrid++)
     for (ygrid=0; ygrid < 2; ygrid++)
     {
     // pick the effect to use
     effectLoc = glGetUniformLocation(p, "whichEffect");
     glUniform1i(effectLoc, counter);
     counter++;
     
     // draw the polygon with the current texture and effect
     
     glBegin( GL_POLYGON );    
     // GL_TEXTURE_RECTANGLE_ARB uses pixel coordinates rather than the standard OpenGL 0.0 to 1.0
     //glTexCoord2f( 0.0f, 1.0f );
     glTexCoord2i(0, showMe->height);
     glVertex3f( -2.0f+2*xgrid, -2.0f+2*ygrid, 0.0f );
     
     //glTexCoord2f( 1.0f, 1.0f ); 
     glTexCoord2i(showMe->width, showMe->height);
     glVertex3f(  0.0f+2*xgrid, -2.0f+2*ygrid, 0.0f );
     
     //glTexCoord2f( 1.0f, 0.0f );
     glTexCoord2i(showMe->width, 0);
     glVertex3f(  0.0f+2*xgrid,  0.0f+2*ygrid, 0.0f );
     
     //glTexCoord2f( 0.0f, 0.0f ); 
     glTexCoord2i(0, 0);
     glVertex3f( -2.0f+2*xgrid,  0.0f+2*ygrid, 0.0f );
     glEnd();
     }*/
	glutSwapBuffers();
}





////////////////////////////////////////////////////////////////////////////////

void processNormalKeys(unsigned char key, int x, int y) {

	// exit program
	if (key == 27) 
		exit(0);
    else if (key =='q'|| key =='Q'){
		which=1;
	}
	else if (key=='z'|| key == 'Z'){
        which =0;}
	else if (key=='w'|| key == 'W'){
        which =2;}
	else if (key=='a'|| key == 'A'){
        which =3;}
	else if (key=='s'|| key == 'S'){
        which =4;}
	else if (key=='1')
	{mode=1;which=0;}
	else if (key=='2')
	{mode=2;which=0;}
}

////////////////////////////////////////////////////////////////////////////////

GLuint setShaders(char * vert, char * frag) {
    GLuint v,f, pro;
	char *vs,*fs;

	v = glCreateShader(GL_VERTEX_SHADER);
	f = glCreateShader(GL_FRAGMENT_SHADER);

	vs = textFileRead(vert);
	fs = textFileRead(frag);

	const char * vv = vs;
	const char * ff = fs;

	glShaderSource(v, 1, &vv,NULL);
	glShaderSource(f, 1, &ff,NULL);

	free(vs);free(fs);

	glCompileShader(v);
	glCompileShader(f);

	printShaderLog(v);
	printShaderLog(f);

	pro = glCreateProgram();
	glAttachShader(pro,v);
	glAttachShader(pro,f);

	glLinkProgram(pro);
	printProgramLog(pro);
	
	return(pro);
}


void bump()
{
    double FPS = CalFrequency();
    // printf("FPS = %f\n", FPS);
    GLint texLoc, effectLoc;//32位整数
    int xgrid, ygrid, counter;
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glLoadIdentity();
    gluLookAt(0.0,0.0,5.0, 
              0.0,0.0,-1.0,
              0.0f,1.0f,0.0f);
    glColor3f(1.0, 1.0, 1.0);	
    
    // openCV stuff
    IplImage *frame = 0;
    
    frame = cvQueryFrame( capture );
    if(!frame) 
    {
        exit(1);
    }
    
    // create a guaranteed texture size
    destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
    cvResize(frame, destination);
    
    // set the texture parameters
    glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    
    showMe = destination;
    
    if(showMe->nChannels == 3)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
    } 
    else if(frame->nChannels == 4)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
    }
    
    glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
    
    // use the shader
    glUseProgram(p);
    GLint texLoc2, lightLoc, bumpSizeLoc, bumpDensityLoc;
    texLoc2 = glGetAttribLocation(p, "Tangent");
    glVertexAttrib3f(texLoc2, 1.0, 0.0, 0.0);
    
    
    
    lightLoc = glGetUniformLocation(p, "LightPosition");
    glUniform3f(lightLoc, a1, a2, a3);
    
    bumpSizeLoc = glGetUniformLocation(p, "BumpSize");
    glUniform1f(bumpSizeLoc, bSize);
    
    bumpDensityLoc = glGetUniformLocation(p, "BumpDensity");
    glUniform1f(bumpDensityLoc, bDensity);
    
    glNormal3f(0.0f, 0.0f, 1.0f );
    
	// always using texture 0 for now
    texLoc = glGetUniformLocation(p, "whichTexture");
    glUniform1i(texLoc, 0);
    
    
	// by default draw a white polygon if the texturing doesn't work.
    glColor3f(1.0, 1.0, 1.0);
    counter = 0;
    
    // draw the textured polygons
    // for some more sophisitcated effects you will probably want to render the texture to an offscreen
    // buffer first to be able to chain multiple shader effects together
    

    // always using texture 0 for now
    texLoc = glGetUniformLocation(p, "whichTexture");
    glUniform1i(texLoc, 0);
    
    
    // by default draw a white polygon if the texturing doesn't work.
    glColor3f(1.0, 1.0, 1.0);
    counter = 0;
    
    // draw the textured polygons
    // for some more sophisitcated effects you will probably want to render the texture to an offscreen
    // buffer first to be able to chain multiple shader effects together
    
    
    effectLoc = glGetUniformLocation(p, "whichEffect");
    glUniform1i(effectLoc, 6);
    glEnable(GL_TEXTURE_2D);
    glColor3f(1.0, 1.0, 1.0);
    
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0f, 1.0f );
    glVertex3f( -2.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 1.0f ); 
    
    glVertex3f(  2.0f, -2.0f, 0.0f );
    
    glTexCoord2f( 1.0f, 0.0f );
    
    glVertex3f(  2.0f,  2.0f, 0.0f );
    
    glTexCoord2f( 0.0f, 0.0f ); 
    glVertex3f( -2.0f,  2.0f, 0.0f );
    glEnd();//map the texture to polygon
    glutSwapBuffers();
        
}

void renderSceneOpt(){
	if (which==1 && mode==1){//enlarge upper left photo
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 0);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);
        glBegin( GL_POLYGON );
        glTexCoord2f( 0.0f, 1.0f );
        glVertex3f( -2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 1.0f ); 
        
        glVertex3f(  2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 0.0f );
        
        glVertex3f(  2.0f,  2.0f, 0.0f );
		
        glTexCoord2f( 0.0f, 0.0f ); 
        glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();
        glutSwapBuffers();
    }
	else if (which==0&& mode==1){//return normal
		renderScene();
        
	}
	else if (which==2&& mode==1){//enlarge the upper right photo
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 1);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);        glBegin( GL_POLYGON );
        glTexCoord2f( 0.0f, 1.0f );
        glVertex3f( -2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 1.0f ); 
        
        glVertex3f(  2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 0.0f );
        
        glVertex3f(  2.0f,  2.0f, 0.0f );
		
        glTexCoord2f( 0.0f, 0.0f ); 
        glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();
        glutSwapBuffers();
        
	}
	else if (which==3&& mode==1){//enlarge lowerleft photo
        
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 2);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);
        
        glBegin( GL_POLYGON );
        glTexCoord2f( 0.0f, 1.0f );
        glVertex3f( -2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 1.0f ); 
        
        glVertex3f(  2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 0.0f );
        
        glVertex3f(  2.0f,  2.0f, 0.0f );
		
        glTexCoord2f( 0.0f, 0.0f ); 
        glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();//map the texture to polygon
        glutSwapBuffers();
        
	}
	else if (which==4&& mode==1){//enlarge lower right
        
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 3);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);
        glBegin( GL_POLYGON );
        glTexCoord2f( 0.0f, 1.0f );
        glVertex3f( -2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 1.0f ); 
        
        glVertex3f(  2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 0.0f );
        
        glVertex3f(  2.0f,  2.0f, 0.0f );
		
        glTexCoord2f( 0.0f, 0.0f ); 
        glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();
        glutSwapBuffers();
        
	}
	else if (which==0&&mode==2)
	{
        if(flag == 0)
        {a3 += 0.3;
            //a1 += 0.3;
            //a2 += 0.3;
            
            if( a3 >= 9.0 )
        {flag = 1;}
            
            
            }
        else
        {a3 -= 0.3;
            //a1 = 0.3;
            //a2 -= 0.3;
        
            if( a3 <= 1.0 )
            {flag = 0;}
        
        
        }
        
        renderScene2();
	}
	else if (which==1 && mode==2){
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 0);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);
        glBegin( GL_POLYGON );	
		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( -2.0f, -2.0f, 0.0f );
		
		glTexCoord2f( 1.0f, 1.0f ); 
        
		glVertex3f(  2.0f, -2.0f, 0.0f );
		
		glTexCoord2f( 1.0f, 0.0f );
        
		glVertex3f(  2.0f,  2.0f, 0.0f );
		
		glTexCoord2f( 0.0f, 0.0f ); 
		glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();//map the texture to polygon
        glutSwapBuffers();
	}
	else if (which==2 && mode==2){
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 5);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);
	    glBegin( GL_POLYGON );
		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( -2.0f, -2.0f, 0.0f );
		
		glTexCoord2f( 1.0f, 1.0f ); 
        
		glVertex3f(  2.0f, -2.0f, 0.0f );
		
		glTexCoord2f( 1.0f, 0.0f );
        
		glVertex3f(  2.0f,  2.0f, 0.0f );
		
		glTexCoord2f( 0.0f, 0.0f ); 
		glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();//map the texture to polygon	
        
        glutSwapBuffers();
	}
	else if (which==3 && mode==2){
        
        if(flag == 0)
        {a3 += 0.3;
            //a1 += 0.3;
            //a2 += 0.3;
            
            if( a3 >= 9.0 )
            {flag = 1;}
            
            
        }
        else
        {a3 -= 0.3;
            //a1 = 0.3;
            //a2 -= 0.3;
            
            if( a3 <= 1.0 )
            {flag = 0;}
            
            
        }
        bump();
	}
	else if(which==4 && mode==2){
	    
        GLint texLoc, effectLoc;//32位整数
        int xgrid, ygrid, counter;
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
        glLoadIdentity();
        gluLookAt(0.0,0.0,5.0, 
                  0.0,0.0,-1.0,
                  0.0f,1.0f,0.0f);
        glColor3f(1.0, 1.0, 1.0);	
        
        // openCV stuff
        IplImage *frame = 0;
        
        frame = cvQueryFrame( capture );
        if(!frame) 
        {
            exit(1);
        }
        
        // create a guaranteed texture size
        destination = cvCreateImage ( cvSize(800, 600), frame->depth, frame->nChannels );
        cvResize(frame, destination);
        
        // set the texture parameters
        glBindTexture( GL_TEXTURE_2D, cameraImageTextureID); //bind the texture to it's array
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        
        showMe = destination;
        
        if(showMe->nChannels == 3)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, showMe->width, showMe->height, 0, GL_BGR, GL_UNSIGNED_BYTE, showMe->imageData);//载入
        } 
        else if(frame->nChannels == 4)
        {
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, showMe->width, showMe->height, 0, GL_BGRA, GL_UNSIGNED_BYTE, showMe->imageData);
        }
        
        glBindTexture(GL_TEXTURE_2D, cameraImageTextureID);
        
        // use the shader
        glUseProgram(p);
        
        // always using texture 0 for now
        texLoc = glGetUniformLocation(p, "whichTexture");
        glUniform1i(texLoc, 0);
        
        
        
        // by default draw a white polygon if the texturing doesn't work.
        glColor3f(1.0, 1.0, 1.0);
        counter = 0;
        
        // draw the textured polygons
        // for some more sophisitcated effects you will probably want to render the texture to an offscreen
        // buffer first to be able to chain multiple shader effects together
        
        
        effectLoc = glGetUniformLocation(p, "whichEffect");
        glUniform1i(effectLoc, 7);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0, 1.0, 1.0);
        glBegin( GL_POLYGON );
        glTexCoord2f( 0.0f, 1.0f );
        glVertex3f( -2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 1.0f ); 
        
        glVertex3f(  2.0f, -2.0f, 0.0f );
		
        glTexCoord2f( 1.0f, 0.0f );
        
        glVertex3f(  2.0f,  2.0f, 0.0f );
		
        glTexCoord2f( 0.0f, 0.0f ); 
        glVertex3f( -2.0f,  2.0f, 0.0f );
        glEnd();
        glutSwapBuffers();
    }
    
}

void MouseClickCallback(int button, int state, int x, int y) {
    
	//upper left relative location to window, switching to
	//lower left relative location to stay consistent with the rest
	//of the code.
	y = winHeight - y;
	if (y <0)
		y = 0;
	if (y>winHeight-1) //start at 0 so windowHeight-1 is the maximum value
		y = winHeight-1;
	//only process mouse up events on left click.
	if ((button==GLUT_LEFT_BUTTON) && (state == GLUT_UP)) {
		// if in full screen mode then get back to all view
		if (mode==1 && which==1) {
			mode=1;which=0; //set it back to view all on page
		}
        else if (mode==1 && which==2) {
			mode=1;which=0; //set it back to view all on page
		}
        else if (mode==1 && which==3) {
			mode=1;which=0; //set it back to view all on page
		}
        else if (mode==1 && which==4) {
			mode=1;which=0; //set it back to view all on page
		}
        else if (mode==2 && which==1) {
			mode=2;which=0; //set it back to view all on page
		}
        else if (mode==2 && which==2) {
			mode=2;which=0; //set it back to view all on page
		}
        else if (mode==2 && which==3) {
			mode=2;which=0; //set it back to view all on page
		}
        else if (mode==2 && which==4) {
			mode=2;which=0; //set it back to view all on page
		}
        
        
        
        else if(mode==1 && which==0){
            //otherwise select a square
			//find out which quadrant
			if (x<winWidth/2) {
				//either q or a
				if (y<winHeight/2) {
					//it is a
					mode=1;which=3;
				}else {
					//it is q
					mode=1;which=1;
				}
			}
			else {
				//either w or s
				if (y<winHeight/2) {
					//it is s
					mode=1;which=4;
				}else {
					//it is w
					mode=1;which=2;
				}
                
			}
            
            
		}
        
        else if(mode==2 && which==0){
            //otherwise select a square
			//find out which quadrant
			if (x<winWidth/2) {
				//either q or a
				if (y<winHeight/2) {
					//it is a
					mode=2;which=3;
				}else {
					//it is q
					mode=2;which=1;
				}
			}
			else {
				//either w or s
				if (y<winHeight/2) {
					//it is s
					mode=2;which=4;
				}else {
					//it is w
					mode=2;which=2;
				}
                
			}
            
            
		}

        
        
        
	}
    
}






////////////////////////////////////////////////////////////////////////////////

int main(int argc, char **argv) {

	// initialize camera or reading from video file
	capture = cvCaptureFromCAM( 0 );
	//capture = cvCaptureFromFile("some file.m4v");  // could also capture from a file


	if( !capture ) 
	{
	printf( "Cannot get access to the video feed (from file or camera)!\n" );
	return 1;
	}

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(winWidth, winHeight);
	glutCreateWindow("SUYU's Project1");

    
	
    glutDisplayFunc(renderScene);
	glutIdleFunc(renderSceneOpt);
	glutReshapeFunc(changeSize);
	glutKeyboardFunc(processNormalKeys);
    glutMouseFunc(MouseClickCallback);

	glewInit();
	
	glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);

	glClearColor(0.0,0.0,0.0,1.0);

 

	if (GLEW_ARB_vertex_shader && GLEW_ARB_fragment_shader)
		printf("Ready for GLSL\n");
	else {
		printf("No GLSL support\n");
		exit(1);
	}

	p = setShaders((char *) "./webcam.vert", (char *) "./webcam.frag");
	
	glGenTextures(1, &cameraImageTextureID);
	glActiveTexture(GL_TEXTURE0 + 0); //GL_TEXTURE0
	    
	glutMainLoop();
	return 0;
}
