uniform int whichEffect;
varying vec2 TexCoord;

varying vec3 LightDir;
varying vec3 EyeDir;

uniform float BumpDensity;
float SpecularFactor = 0.5;

uniform float BumpSize;

uniform sampler2D whichTexture;

void main(void)
{
   vec4 sum = vec4(0.0);
   vec4 part = vec4(0.0);
   
    
if (whichEffect == 0)
{		//original
    part = texture2D(whichTexture, TexCoord.xy);
    sum = part;	
}
    
if (whichEffect == 1)
{	
	vec2 texCoord;
    vec2 normCoord = 2.0 * TexCoord - 1.0; // [-1.0 ,1.0] x [-1.0, 1.0]

	float r = length(normCoord); // to polar coords
	float phi = atan(normCoord.y, normCoord.x); // to polar coords
    
    normCoord.x =  normCoord.x * sign(normCoord.x); // mirror

	texCoord = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]
	sum = texture2D(whichTexture, texCoord);


}

    if (whichEffect == 3) {
        
        vec2 TexSize = vec2(800,600);

        vec2  upLeftUV = vec2(TexCoord.x - 1.0/TexSize.x , TexCoord.y - 1.0/TexSize.y);
        vec4  bkColor = vec4(1.0 , 1.0 , 1.0 , 1.0);
        vec4  curColor    =  texture2D( whichTexture, TexCoord.xy );
        vec4  upLeftColor =  texture2D( whichTexture, upLeftUV );
        
        vec4  delColor1 = curColor-upLeftColor ;
        vec4  delColor2 = upLeftColor-curColor;
        vec4 delColor = (delColor1 + (-delColor2))*100000.0;
        float  h = 0.3 * delColor.x + 0.59 * delColor.y + 0.11* delColor.z;
        sum =  vec4(h,h,h,0.0)+ bkColor ;
        
        
    }
    
    


if (whichEffect == 2)
{	//mosaic
vec2 intxy;
   intxy=vec2(512.0*TexCoord.s,512.0*TexCoord.t);
        vec2 mosaicxy;
        vec2 access;
        vec3 color;
        mosaicxy=vec2(int(intxy.x/32.0)*32,int(intxy.y/32.0)*32);
        access=vec2(mosaicxy.x/512.0,mosaicxy.y/512.0);
        color=vec3(texture2D(whichTexture,access)); 
        sum=vec4(color,1.0);
		
}

if (whichEffect == 5)
    {
//chromatic aberration
        float scale=0.8;            

        //texture coordinates
        vec2 texcoord = gl_TexCoord[0].st;
        // get the right pixel for the current position
            
        vec2 rCoords = 1.09*scale*(texcoord.xy);//causing chromatic dispersion
        vec2 gCoords = 1.06*scale*(texcoord.xy);
        vec2 bCoords = 1.03*scale*(texcoord.xy);            
        vec3 inputDistort = vec3(0.0); 
        
        inputDistort.r = texture2D(whichTexture,rCoords).r;
        inputDistort.g = texture2D(whichTexture,gCoords).g;
        inputDistort.b = texture2D(whichTexture,bCoords).b;
        
        sum = vec4(inputDistort.r,inputDistort.g,inputDistort.b,1.0);
        }
    
    if (whichEffect == 6)
    {//bump mapping
        vec3 litColor;
        vec2 c = BumpDensity * gl_TexCoord[0].st;
        vec2 p = fract(c) - vec2 (0.5);
        vec3 texel=vec3(texture2D(whichTexture,gl_TexCoord[0].st));

        float d, f;
        d = p.x * p.x + p.y * p.y;
        f = 1.0 / sqrt(d + 1.0);
        
        // if we are outside the area of a bump then
        // the normal points straight out Z w/ length 1
        if (d >= BumpSize)
		{
            p = vec2(0.0);
            f = 1.0;
		}
        
        // create a new normal of length 1 for this fragment
        vec3 normDelta = vec3 (p.x, p.y, 1.0) * f;
        
        litColor = texel * max(dot(normDelta, LightDir), 0.0);
        vec3 reflectDir = reflect(LightDir, normDelta);
        
        float spec = max(dot(EyeDir, reflectDir), 0.0);
        spec = pow(spec, 6.0);
        spec *= SpecularFactor;
        litColor = min(litColor + spec, vec3 (1.0));
        
        sum = vec4 (litColor, 1.0);

    }
    
    if (whichEffect == 7)
    {	
// fish eye 
        vec2 texCoord;
        vec2 normCoord = 2.0 * TexCoord - 1.0; // [-1.0 ,1.0] x [-1.0, 1.0]
        
        float r = length(normCoord); // to polar coords
        float phi = atan(normCoord.y, normCoord.x); // to polar coords
        

        r = r * r / sqrt(2.0); 
        
        normCoord.x = r * cos(phi);
        normCoord.y = r * sin(phi);
        
        
        texCoord = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]
        sum = texture2D(whichTexture, texCoord);
        
        
    }
    
    
    gl_FragColor = sum;
}